__version__ = '0.0.15'

"""

Sample settings

BLOXBY_BUILDER = {
    'url': 'http://clouddigitalmarketing.com',
    'username': 'hello@linkfusions.com',
    'password': 'accountpassword',
    'package_prefix': 'LF-',
    'account_email_prefix': 'LF',
    'public_key': 'Yourpublickeyasstatedinthebloxbyapplicationsettings',
    'autologin_hash': 'Yourautologinhash'
}

account_email_prefix: Prepended to all the emails when registering the account, just to make sure the user can use
same email for more than one app (i.e. Can maintain different accounts connected to linkfusions and also connected to
vmm at the same time).
Same applies to package_prefix for packages

"""



from django.conf import settings
from django.db import models

from .managers import UserBridgeManager
from .functions import Bloxby

bloxby = Bloxby()


class UserBridge(models.Model):
    """You can create this at post_save of PurchasedPackage or something similar, a model that represents that the user
    has purchased a package, and delete it also at post_delete"""
    user = models.OneToOneField(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    bloxby_id = models.IntegerField(blank=True, null=True)
    autologin_token = models.CharField(max_length=20, blank=True, null=True)
    active = models.BooleanField(default=True)
    created = models.DateTimeField(auto_now_add=True)

    objects = UserBridgeManager()

    def __str__(self):
        return f'{self.user.email} - {self.bloxby_id}'

    @property
    def dashboard_url(self):
        if self.active:
            url = settings.BLOXBY_BUILDER['url']
            token = self.autologin_token
            hash_string = bloxby.generate_login_hash()
            return f"{url}/auth/alogin/?token={token}&hash={hash_string}"
        return '#'

    def delete(self, using=None, keep_parents=False):
        bloxby.Users.update(self.bloxby_id, status='Inactive')
        self.active = False
        self.save()

    @classmethod
    def create_remote(cls, user, package_id, user_bridge):
        data, success = bloxby.Users.create(
            user.first_name, user.last_name, user.email, bloxby.generate_random_password(), package_id
        )
        if success:
            user_bridge.bloxby_id = data['data']['user']['id']
            user_bridge.autologin_token = data['data']['user']['auto_login_token']
            user_bridge.save()
        return data, success

    @classmethod
    def create(cls, user, package_id=None):
        if not package_id:
            package_id = settings.BLOXBY_BUILDER['default_package_id']
        """
        Package_id is the bloxby package id not the internal one
        Simply call UserBridge.create(package_id, user);
        """
        # Check if user already exists in bloxby
        existings = cls.objects.everything().filter(user=user)
        if existings.exists():
            obj = existings.first()
            obj.active = True
            obj.save()
            new = obj
            data, is_success = bloxby.Users.update(obj.bloxby_id, status='Active')
            if not is_success and 'does not belong to a valid user' in str(data):
                cls.create_remote(user, package_id, obj)
        else:
            # Check if email already exists there
            new = cls.objects.create(user=user, override=True)
            data, success = cls.create_remote(user, package_id, new)
            if not success:
                if 'email already used' in str(data):
                    # Hack to find the user again. Not efficient but we have to work with what we have
                    bloxby_users, _ = bloxby.Users.all()
                    for bu in bloxby_users:
                        if user.email in bu['email']:
                            new.bloxby_id = bu['id']
                            new.autologin_token = bu['auto_login_token']
                            new.save()
                            break
        return new



class Generic:
    def __init__(self, base_url, username, password, package_prefix, public_key, account_email_prefix):
        self.base_url = base_url
        self.username = username
        self.password = password
        self.package_prefix = package_prefix
        self.account_email_prefix = account_email_prefix
        self.public_key = public_key

    @staticmethod
    def bool_cast(b):
        return 'yes' if b else 'no'

    def get_auth(self):
        return self.username, self.password

    def get_headers(self):
        return {
            'X-API-KEY': self.public_key,
        }

    def create(self, **kwargs):
        pass

    def update(self, **kwargs):
        pass

    def delete(self, **kwargs):
        pass

    def retrieve(self, **kwargs):
        pass



from django import template

from ..models import UserBridge

register = template.Library()


@register.simple_tag(takes_context=True)
def user_builder_dashboard(context):
    user = context['request'].user
    if user.is_authenticated:
        try:
            return user.userbridge.dashboard_url
        except UserBridge.DoesNotExist:
            pass
    return '#'



import requests

from .generics import Generic


class User(Generic):
    path = '/api/users/'

    def create(self, first_name, last_name, email, password, package_id=None, type_='User'):
        data = {
            'first_name': first_name,
            'last_name': last_name,
            'email': f'{self.account_email_prefix}{email}',
            'password': password,
            'type': type_,
            'package_id': package_id
        }
        response = requests.post(
            f'{self.base_url}{self.path}', data=data, headers=self.get_headers(), auth=self.get_auth()
        )
        return response.json(), response.status_code in [200, 201]

    def update(self, user_id, **kwargs):
        response = requests.put(
            f'{self.base_url}{self.path}{user_id}/', data=kwargs, headers=self.get_headers(), auth=self.get_auth()
        )
        return response.json(), response.status_code in [201, 200]

    def delete(self, user_id):
        response = requests.delete(
            f'{self.base_url}{self.path}{user_id}/', headers=self.get_headers(), auth=self.get_auth()
        )
        return response.status_code == 204

    def retrieve(self, user_id):
        response = requests.get(
            f'{self.base_url}{self.path}{user_id}/', headers=self.get_headers(), auth=self.get_auth()
        )
        data = response.json()
        is_success = response.status_code == 200
        return data[0] if is_success else data, is_success

    def all(self):
        response = requests.get(
            f'{self.base_url}{self.path}/', headers=self.get_headers(), auth=self.get_auth()
        )
        data = response.json()
        is_success = response.status_code == 200
        return data if is_success else data, is_success
